<?php
// $Horde: imp/lib/folder.php,v 1.28 2001/01/04 18:34:12 jon Exp $

/**
 * The IMP_folder:: class provides a set of methods for dealing with
 * folders, accounting for subscription, errors, etc.
 *
 * @author   Chuck Hagenbuch <chuck@horde.org>
 * @author   Jon Parise <jon@csh.rit.edu>
 * @version  $Revision: 1.28 $
 * @since    IMP 2.3
 */
class IMP_folder {
    
    /**
     * List folders.
     *
     * @param $stream      An open connection to a mail server.
     *
     * @param $server      A string representing the specification of
     *                     the server to connect to.
     *                     Ex: {mail.foo.com/pop3:110}.
     *
     * @param $prefix      What to restrict the folder list to. For
     *                     instance, if the IMAP server makes your
     *                     entire * home directory * available, but
     *                     you only want * the folders in ~/mail/,
     *                     then this should be * 'mail/'.
     *
     * @param $subscribed  (optional) Should we list only subscribed folders?
     *
     * @param $filter      (optional) An array containing mailboxes that should be
     *                     left out of the list.
     *
     * @param $hierarchies (optional) An array containing additional
     *                     hierarchies that are either outside prefix
     *                     or not advertised by * default (#shared/
     *                     and similar with UW) to include in the
     *                     list.
     *
     * @param $dotfiles    (optional) If false, don't include folders
     *                     beginning with '.'
     *
     * @param $namespace   (optional) Namespace prefix prepended on folder
     *                     names (eg. "INBOX." in cyrus).  We remove this
     *                     for cosmetic purposes.
     *
     * @return An array of folders, where each array alement is an
     *         associative array containing three values: 'val', with
     *         entire folder name after the server specification;
     *         'label', with the full-length folder name meant for
     *         display ($prefix has been stripped off, etc); and
     *         'abbrev', containing a shortened (26 characters max)
     *         label for display in situations where space is short.
     */
    function flist ($stream, $server, $prefix, $subscribed = false, $filter = array(), $hierarchies = array(), $dotfiles = true, $namespace = '') {
        // keep around identical lists so that we don't hit the server
        // more that once in the same page for the same thing.
        static $list_cache;
        
        // keep track of mailbox names that we have complained about
        // so as not to give the user identical error messages.
        static $error_cache;
        
        // only initialize the static caches if this is the first time
        // flist() has been called.
        if (!isset($list_cache)) $list_cache = array();
        if (!isset($error_cache)) $error_cache = array();
        
        // check parameters
        if (!is_array($filter)) $filter = array();
        if (!is_array($hierarchies)) $hierarchies = array();
        
        // make sure we're requesting all folders.
        $folders = $prefix . '*';
        
        // figure out which list command to use.
        if ($subscribed) {
            $listcmd = 'imap_listsubscribed';
        } else {
            $listcmd = 'imap_listmailbox';
        }
        
        // compute a value that will uniquely identify this list
        $signature = implode('|', array($listcmd, $folders));
        
        // either get the list from the cache, or go to the IMAP
        // server to get it if we haven't gotten it yet.
        if (isset($list_cache[$signature])) {
            $mailboxes = $list_cache[$signature];
        } else {
            $mailboxes = array();
            $mailboxes = $listcmd($stream, $server, $folders);
            
            foreach ($hierarchies as $hierarchy) {
                $hboxes = $listcmd($stream, $server, $hierarchy . '*');
                if (is_array($hboxes)) {
                    $mailboxes = array_merge($mailboxes, $hboxes);
                }
            }
            
            $list_cache[$signature] = $mailboxes;
        }
        
        $list = array();
        if (is_array($mailboxes) && (count($mailboxes) > 0)) {
            // these only need to be assigned once
            $junk = $server . $prefix;
            
            sort($mailboxes);
            $seen = array();
            
            while (list(,$mailbox) = each($mailboxes)) {
                if (empty($seen[$mailbox])) {
                    $seen[$mailbox] = true;
                    if (!($decoded_mailbox = @imap_utf7_decode($mailbox))) {
                        if (empty($error_cache[$mailbox])) {
                            $user_box = str_replace($server, '', $mailbox);
                            $user_box = (strlen($prefix) > 0) ? str_replace($prefix, '', $user_box) : $user_box;
                            if (!empty($namespace) && strstr($user_box, $namespace)) {
                                $user_box = str_replace($namespace, '', $user_box);
                            }
                            Horde::raiseMessage(sprintf(_("The folder \"%s\" contains illegal characters in its name. It may cause problems. Please see your system administrator."), $user_box, imap_utf7_encode($user_box)), HORDE_WARNING);
                            $error_cache[$mailbox] = true;
                        }
                    } else {
                        $mailbox = $decoded_mailbox;
                    }
                    
                    // ignore INBOX, since we always tack it on first
                    if (strcmp($junk . 'INBOX', $mailbox) != 0) {
                        $value = str_replace($server, '', $mailbox);
                        $label = (strlen($prefix) > 0) ? str_replace($prefix, '', $value) : $value;
                        // Strip off the namespace string, if it exists.
                        if (!empty($namespace) && strstr($label, $namespace)) {
                            $label = str_replace($namespace, '', $label);
                        }
                        
                        if (strlen($label) > 26) {
                            $abbrev = substr($label, 0, 10) . '...' . substr($label, -13, 13);
                        } else {
                            $abbrev = $label;
                        }
                        if (($dotfiles || ($label[0] != '.'))
                            && !in_array($value, $filter)) {
                            $list[$value] = array('val' => $value, 'label' => $label, 'abbrev' => $abbrev);
                        }
                    }
                }
            }
        }
        
        if (!in_array('INBOX', $filter)) {
            array_unshift($list, array('val' => 'INBOX', 'label' => 'INBOX', 'abbrev' => 'INBOX'));
        }
        
        return $list;
    }
    
    /**
     * List all folders which the user is not subscribed to.
     *
     * @param $stream      An open connection to a mail server.
     *
     * @param $server      A string representing the specification of
     *                     the server to connect to.
     *                     Ex: {mail.foo.com/pop3:110}.
     *
     * @param $prefix      What to restrict the folder list to. For
     *                     instance, if the IMAP server makes your
     *                     entire * home directory * available, but
     *                     you only want * the folders in ~/mail/,
     *                     then this should be * 'mail/'.
     *
     * @param $filter      (optional) An array containing mailboxes that should be
     *                     left out of the list.
     *
     * @param $hierarchies (optional) An array containing additional
     *                     hierarchies that are either outside prefix
     *                     or not advertised by * default (#shared/
     *                     and similar with UW) to include in the
     *                     list.
     *
     * @param $dotfiles    (optional) If false, don't include folders
     *                     beginning with '.'
     *
     * @param $namespace   (optional) Namespace prefix prepended on folder
     *                     names (eg. "INBOX." in cyrus).  We remove this
     *                     for cosmetic purposes.
     *
     * @return An array of folders, in the same format as IMP_folder::flist().
     */
    function flistUnsubscribed ($stream, $server, $prefix, $filter = false, $hierarchies = false, $dotfiles = true, $namespace = '') {
        $sub = IMP_folder::flist($stream, $server, $prefix, true, $filter, $hierarchies, $dotfiles, $namespace);
        $all = IMP_folder::flist($stream, $server, $prefix, false, $filter, $hierarchies, $dotfiles, $namespace);
        
        $usub = array();
        foreach ($all as $key => $val) {
            if (empty($sub[$key]))
                $usub[$key] = $all[$key];
        }
        
        return $usub;
    }
    
    /**
     * Delete one or more folders.
     * 
     * @param $stream a valid c-client stream
     * @param $folder_array an array of folders to be deleted
     * @param $subscribe a boolean describing whether or not to use
     *                   folder subscriptions
     *
     * @return boolean Whether or not the folders were successfully deleted.
     */
    function delete ($stream, $folder_array, $subscribe) {
        $server = serverString();
        $return_value = true;
        if ($subscribe) {
            $subscribed_folders = imap_listsubscribed($stream, $server, '*');
        }
        
        foreach ($folder_array as $folder) {
            if (!imap_deletemailbox($stream, $server . imap_utf7_encode($folder))) {
                Horde::raiseMessage(sprintf(_("The folder \"%s\" was not deleted. This is what the server said"), IMP_folder::displayName($folder)) .
                                    ': <i>' . imap_last_error() . '</i>', HORDE_ERROR);
                $return_value = false;
            } elseif ($subscribe && in_array($server . $folder, $subscribed_folders) &&
                      !imap_unsubscribe($stream, $server . imap_utf7_encode($folder))) {
                Horde::raiseMessage(sprintf(_("The folder \"%s\" was deleted but you were not unsubscribed from it."), IMP_folder::displayName($folder)), HORDE_WARNING);
                $return_value = false;
            } else {
                Horde::raiseMessage(sprintf(_('The folder "%s" was successfully deleted.'), IMP_folder::displayName($folder)), HORDE_SUCCESS);
            }
        }
        
        return $return_value;
    }
    
    /**
     * Create a new IMAP folder if it does not already exist, and
     * subcribe to it as well if requested.
     *
     * @param $stream a valid c-client stream
     * @param $folder the full name of the folder to be created
     * @param $subscribe a boolean saying whether or not to use folder
     *                   subscriptions
     *
     * @return boolean Whether or not the folder was successfully created.
     */
    function create ($stream, $folder, $subscribe) {
        /* Make sure we are not trying to create a duplicate folder */
        if (IMP_folder::exists($stream, $folder)) {
            Horde::raiseMessage(sprintf(_("The folder \"%s\" already exists"), IMP_folder::displayName($folder)), HORDE_WARNING);
            return false;
        } 

        /* Attempt to create the mailbox */
        if (!imap_createmailbox($stream, serverString() . imap_utf7_encode($folder))) {
            Horde::raiseMessage(sprintf(_("The folder \"%s\" was not created. This is what the server said"), IMP_folder::displayName($folder)) .
                            ': <i>' . imap_last_error() . '</i>', HORDE_ERROR);
            return false;
        } 

        /* If the user uses SUBSCRIBE, then add to the subscribe list */
        if ($subscribe && !imap_subscribe($stream, serverString() . imap_utf7_encode($folder))) {
            Horde::raiseMessage(sprintf(_("The folder \"%s\" was created but you were not subscribed to it."), IMP_folder::displayName($folder)), HORDE_WARNING);
            return false;
        }
      
        /* The folder creation has been successful */ 
        Horde::raiseMessage(sprintf(_('The folder "%s" was successfully created.'), IMP_folder::displayName($folder)), HORDE_SUCCESS); 
        return true;
    }
    
    /**
     * Find out if a specific folder exists or not.
     *
     * @param $stream a valid c-client stream
     * @param $folder the full folder name to be checked.
     *
     * @return boolean Whether or not the folder exists
     */
    function exists ($stream, $folder) {
        $result = imap_listmailbox($stream, '', serverString() . imap_utf7_encode($folder));
        return is_array($result);
    }
    
    /**
     * Rename an IMAP folder.
     * 
     * @param $stream a valid c-client stream
     * @param $old the old folder name
     * @param $new the new folder name
     * @param $subscribe a boolean saying whether or not to use
     *                   folder subscriptions
     *
     * @return boolean Whether or not the folder was successfully renamed.
     */
    function rename ($stream, $old, $new, $subscribe) {
        $server = serverString();
        if ($subscribe) {
            $subscribed_folders = imap_listsubscribed($stream, $server, '*');
        }
        
        if (imap_renamemailbox($stream, $server . imap_utf7_encode($old), $server . imap_utf7_encode($new))) {
            if ($subscribe && in_array($server . $old, $subscribed_folders)) {
                imap_unsubscribe($stream, $server . imap_utf7_encode($old));
                imap_subscribe($stream, $server . imap_utf7_encode($new));
            }
            Horde::raiseMessage(sprintf(_('The folder "%s" was successfully renamed to "%s".'), IMP_folder::displayName($old), IMP_folder::displayName($new)), HORDE_SUCCESS);
            return true;
        } else {
            Horde::raiseMessage(sprintf(_("Renaming \"%s\" to \"%s\" failed. This is what the server said"), IMP_folder::displayName($old), IMP_folder::displayName($new)) .
                                ' : <i>' . imap_last_error() . '</i>', HORDE_ERROR);
            return false;
        }
    }
    
    /**
     * Subscribe to one or more IMAP folders.
     * 
     * @param $stream a valid c-client stream
     * @param $folder_array an array of folders to be subscribed
     *
     * @return boolean Whether or not the folders were successfully
     *                 subscribed to.
     */
    function subscribe ($stream, $folder_array) {
        $return_value = true;
        
        if (is_array($folder_array)) {
            foreach ($folder_array as $folder) {
                if ($folder != ' ') {
                    if (!imap_subscribe($stream, serverString() . $folder)) {
                        Horde::raiseMessage(sprintf(_('You were not subscribed to "%s". Here is what the server said'), IMP_folder::displayName($folder)) .
                                        ' :<i>' . imap_last_error() . '</i>', HORDE_ERROR);
                        $return_value = false;
                    } else {
                        Horde::raiseMessage(sprintf(_('You were successfully subscribed to "%s"'), IMP_folder::displayName($folder)), HORDE_SUCCESS);
                    }
                }
            }
        } else {
            Horde::raiseMessage(_("No folders were specified"), HORDE_MESSAGE);
            $return_value = false;
        }
        
        return $return_value;
    }
    
    /**
     * Unsubscribe from one or more IMAP folders.
     *
     * @param $stream a valid c-client stream
     * @param $folder_array an array of folders to be deleted
     *
     * @return boolean Whether or not the folders were successfully
     *                 unsubscribed from.
     */
    function unsubscribe ($stream, $folder_array) {
        $return_value = true;
        
        if (is_array($folder_array)) {
            foreach ($folder_array as $folder) {
                if ($folder != ' ') {
                    if (!imap_unsubscribe($stream, serverString() . $folder)) {
                        Horde::raiseMessage(sprintf(_('You were not unsubscribed from "%s". Here is what the server said'), IMP_folder::displayName($folder)) .
                                        ' :<i>' . imap_last_error() . '</i>', HORDE_ERROR);
                        $return_value = false;
                    } else {
                        Horde::raiseMessage(sprintf(_('You were successfully unsubscribed from "%s"'),IMP_folder::displayName($folder)), HORDE_SUCCESS);
                    }
                }
            }
        } else {
            Horde::raiseMessage(_("No folders were specified"), HORDE_MESSAGE);
            $return_value = false;
        }
        
        return $return_value;
    }
    
    /**
     * If there is information available to tell us about a prefix in
     * front of mailbox names that shouldn't be displayed to the user,
     * then use it to strip that prefix out.
     *
     * @param string $folder The folder name to display.
     *
     * @return string The folder, with any prefix gone.
     */
    function displayName($folder) {
        if (function_exists('preambleString')) {
            $prefix = preambleString();
            if (substr($folder, 0, strlen($prefix)) == $prefix) {
                $folder = substr($folder, strlen($prefix));
            }
        }
        
        return $folder;
    }
    
}
?>
