<?php
// $Horde: imp/lib/message.php,v 1.19 2000/12/02 22:04:31 chuck Exp $

/**
 * The IMP_message:: class contains all functions related to handling
 * messages within IMP, and should (right now this is a design goal,
 * not a reality) be usable outside IMP as well. Actions such as
 * moving, copying, and deleting messages are handled in here so that
 * code need not be repeated between mailbox, message, and other
 * pages.
 *
 * @author   Chris Hyde <chris@jeks.net>
 * @author   Chuck Hagenbuch <chuck@horde.org>
 * @version  $Revision: 1.19 $
 * @since IMP 2.3
 */
class IMP_message {
    
    /**
     * Returns the current page of messages based on index.
     * 
     * @param $arr the whole array of message indices
     * @param $index the current starting point
     * @param $offset is set to the start of the returned section in the list
     * @param $beginIndex is set to the index of $index in the returned section
     * 
     * @return the desired section of the array
     */
    function range($arr, $index, &$offset, &$beginIndex, &$num)
    {
        global $prefs;
        
        // store the original number of messages
        $num = count($arr);
        
        $count = max($prefs->getValue('max_msgs') * 3, 1000);
        $half = $count / 2;
        if ($index < $half) {
            $start = 0;
        } elseif (($index + $count) >= count($arr)) {
            $start = max( 0, count($arr) - $count);
        } else {
            $start = $index - $half;
        }
        
        $end = min($start + $count, count($arr));
        $new = array();
        for ($i = $start; $i < $end; $i++) {
            $new[] = $arr[$i];
        }
        $offset = $start;
        $beginIndex = $index - $start - 1;
        
        return $new;
    }
    
    /**
     * Copies or moves an array of messages to a new folder. Handles
     * use of the '**search' folder and the Trash folder.
     * 
     * @param $targetMbox the mailbox to move/copy messages to
     * @param $msgs the array of messages to be copied or moved
     * @param $action either MOVE_MESSAGES or COPY_MESSAGES
     */
    function copy($targetFolder, $msgs, $action)
    {
        global $imp, $conf;
        
        $targetMbox = imap_utf7_encode($targetFolder);
        $return_value = true;
        $serverstring = serverString();
        $nummsgs = count($msgs);
        
        switch ($action) {
        case MOVE_MESSAGES:
            $imap_flags = CP_UID|CP_MOVE;
            $message = _("There was an error moving messages to \"%s\". This is what the server said");
            break;
        case COPY_MESSAGES:
            $imap_flags = CP_UID;
            $message = _("There was an error copying messages to \"%s\". This is what the server said");
            break;
        }
        
        if ($imp['mailbox'] == '**search') {
            $current_folder = '';
            $msg_list = '';
            for ($i = 0; $i < $nummsgs; $i++) {
                /* In search we have msgnum:folder e.g. 12:Sent */
                $msginfo = explode(':', $msgs[$i]);
                $msg_num = $msginfo[0];
                $msg_folder = $msginfo[1];
                if ($current_folder != $msg_folder) {
                    if ($msg_list != '') {
                        if (!imap_mail_copy($imp['stream'], $msg_list, $targetMbox, $imap_flags)) {
                            Horde::raiseMessage(sprintf($message, IMP_folder::displayName($targetFolder)) . 
                                                ': <i>' . imap_last_error() . '</i>', HORDE_ERROR);
                            $return_value = false;
                        }
                        if ($conf['server']['expunge_on_move'] && $action == MOVE_MESSAGES)
                            imap_expunge($imp['stream']);
                    }
                    $msg_list = $msg_num;
                    $open = imap_reopen($imp['stream'], $serverstring . $msg_folder);
                } else {
                    $msg_list .= ',' . $msg_num;
                }
                $current_folder = $msg_folder;
            }
            /* Tidy up the last few that haven't been copied/moved yet .. */
            if ($msg_list != '') {
                if (!imap_mail_copy($imp['stream'], $msg_list, $targetMbox, $imap_flags)) {
                    Horde::raiseMessage(sprintf($message, IMP_folder::displayName($targetFolder)) . 
                                        ': <i>' . imap_last_error() . '</i>', HORDE_ERROR);
                    $return_value = false;
                }
                if ($conf['server']['expunge_on_move'] && $action == MOVE_MESSAGES) { 
                    imap_expunge($imp['stream']);
                }
            }
        }
        else {
            $msg_list = implode($msgs, ',');
            if (!imap_mail_copy($imp['stream'], $msg_list, $targetMbox, $imap_flags)) {
                Horde::raiseMessage(sprintf($message, IMP_folder::displayName($targetFolder)) . 
                                    ': <i>' . imap_last_error() . '</i>', HORDE_ERROR);
                $return_value = false;
            }
            if ($conf['server']['expunge_on_move'] && $action == MOVE_MESSAGES)
                imap_expunge($imp['stream']);
        }
        
        return $return_value;
    }
    
    /**
     * Deletes an array of messages, taking into account whether or
     * not a Trash folder is being used.
     * 
     * @param $indices the array of messages to be deleted
     */
    function delete ($indices) {
        global $imp, $conf, $prefs;
        
        $return_value = true;
        $serverstring = serverString();
        $trash_folder = preambleString() . $prefs->getValue('trash_folder');
        $msg_list = implode(',', $indices);
        
        if ($imp['mailbox'] == '**search') {
            $old_msg_folder = ' ';
            for ($j = 0; $j < count($indices); $j++) {
                $parts = explode(':',$indices[$j]);
                $msg = $parts[0];
                $msg_folder = $parts[1];
                
                /* If the folder we are deleting from has changed ...*/
                if ($msg_folder != $old_msg_folder) {
                    if ($j > 0)    {
                        /* We're changing folder, we've marked a bunch
                         * of messages deleted, or we've moved them to
                         * the trash. NB: Not expunge_on_move as we
                         * are in the delete func. */
                        if ($conf['server']['use_trash']) {
                            imap_expunge($imp['stream']);
                        }
                    }
                    $old_msg_folder = $msg_folder;
                    $open = imap_reopen($imp['stream'], $serverstring . $msg_folder);
                }
                if ($imp['stream']) {
                    if ($conf['server']['use_trash'] && $imp['mailbox'] != $trash_folder) {
                        include_once './lib/folder.php';
                        if (!IMP_folder::exists($imp['stream'], $trash_folder)) {
                            IMP_folder::create($imp['stream'], $trash_folder, $prefs->getValue('subscribe'));
                        }
                        if (!imap_mail_copy($imp['stream'], $msg, $trash_folder, CP_UID | CP_MOVE)) {
                            Horde::raiseMessage(sprintf(_("There was an error moving messages to the trash. This is what the server said")) .
                                                ': <i>' . imap_last_error() . '</i>', HORDE_ERROR);
                            $return_value = false;
                        }
                    } else {
                        if (!imap_delete($imp['stream'], imap_msgno($imp['stream'], $msg))) {
                            Horde::raiseMessage(sprintf(_("There was an error deleting messages from the folder \"%s\". This is what the server said"),
                                                        IMP_folder::displayName($msg_folder)) . ': <i>' . imap_last_error() . '</i>', HORDE_ERROR);
                            $return_value = false;
                        }
                    }
                }
            }
            if ($conf['server']['use_trash']) {
                imap_expunge($imp['stream']);
            }
        }
        else {
            if ($conf['server']['use_trash']) {
                /* Then move it to the trash folder, and expunge from the original folder */
                if ($imp['mailbox'] != $trash_folder) {
                    include_once './lib/folder.php';
                    if (!IMP_folder::exists($imp['stream'], $trash_folder)) {
                        IMP_folder::create($imp['stream'], $trash_folder, $prefs->getValue('subscribe'));
                    }
                    if (!imap_mail_copy($imp['stream'], $msg_list, $trash_folder, CP_UID | CP_MOVE)) {
                        Horde::raiseMessage(sprintf(_("There was an error moving messages to the trash. This is what the server said")) .
                                            ': <i>' . imap_last_error() . '</i>', HORDE_ERROR);
                        $return_value = false;
                    }
                } else {
                    for ($i = 0; $i < count($indices); $i++) {
                        if (!imap_delete($imp['stream'], imap_msgno($imp['stream'], $indices[$i]))) {
                            Horde::raiseMessage(sprintf(_("There was an error deleting messages. This is what the server said")) .
                                                ': <i>' . imap_last_error() . '</i>', HORDE_ERROR);
                            $return_value = false;
                        }
                    }
                }
                imap_expunge($imp['stream']);
            } else {
                for ($i = 0; $i < count($indices); $i++) {
                    if (!imap_delete($imp['stream'], imap_msgno($imp['stream'], $indices[$i]))) {
                        Horde::raiseMessage(sprintf(_("There was an error deleting messages. This is what the server said")) .
                                            ': <i>' . imap_last_error() . '</i>', HORDE_ERROR);
                        $return_value = false;
                    }
                }
                if ($imp['protocol'] == 'pop3') {
                    imap_expunge($imp['stream']);
                    imap_close($imp['stream']);
                    IMP::authenticate();
                }
            }
        }
        
        return $return_value;
    }
    
    /**
     * Undeletes an array of messages.
     * 
     * @param $msgs the array of message indices to be undeleted
     */
    function undelete ($msgs) {
        global $imp;
        
        $return_value = true;
        $serverstring = serverString();
        
        if ($imp['mailbox'] == '**search') {
            $current_folder = '';
            for ($i = 0; $i < count($msgs); $i++ ) {
                $msginfo = explode(':',$msgs[$i]);
                $msg_num = $msginfo[0];
                $msg_folder = $msginfo[1];
                if ($current_folder != $msg_folder) {
                    $open = imap_reopen($imp['stream'], $serverstring . $msg_folder);
                }
                if ($open && !imap_undelete($imp['stream'], imap_msgno($imp['stream'], $msg_num))) {
                    Horde::raiseMessage(sprintf(_("There was an error deleting messages in the folder \"%s\". This is what the server said"),
                                                IMP_folder::displayName($msg_folder)) . ': <i>' . imap_last_error() . '</i>', HORDE_ERROR);
                    $return_value = false;
                }
                $current_folder = $msg_folder;
            }
        } else {
            for ($i = 0; $i < count($msgs); $i++) {
                if (!(imap_undelete($imp['stream'], imap_msgno($imp['stream'], $msgs[$i])))) {
                    Horde::raiseMessage(sprintf(_("There was an error undeleting messages. This is what the server said")) .
                                        ': <i>' . imap_last_error() . '</i>', HORDE_ERROR);
                    $return_value = false;
                }
            }
        }
        
        return $return_value;
    }
    
    /**
     * Sets or clears a given flag on an array of messages. Handles
     * use of the '**search' folder.
     *
     * Valid flags are:
     *   \\SEEN
     *   \\FLAGGED
     *   \\ANSWERED
     *   \\DELETED
     *   \\DRAFT
     * 
     * @param $flag the imap flag to set or clear
     * @param $msgs the array of messages to be flagged
     * @param boolean $action true: set the flag; false: clear the flag
     */
    function flag($flag, $msgs, $action = true)
    {
        global $imp, $conf;
        
        $return_value = true;
        $nummsgs = count($msgs);
        $serverstring = serverString();
        
        if ($action) {
            $function = 'imap_setflag_full';
        } else {
            $function = 'imap_clearflag_full';
        }
        
        if ($imp['mailbox'] == '**search') {
            $current_folder = '';
            $msg_list = '';
            for ($i = 0; $i < $nummsgs; $i++) {
                /* In search we have msgnum:folder e.g. 12:Sent */
                $msginfo = explode(':', $msgs[$i]);
                $msg_num = $msginfo[0];
                $msg_folder = $msginfo[1];
                if ($current_folder != $msg_folder) {
                    if ($msg_list != '') {
                        if (!$function($imp['stream'], $msg_list, $flag, ST_UID)) {
                            Horde::raiseMessage(sprintf(_("There was an error flagging messages in the folder \"%s\". This is what the server said"),
                                                        IMP_folder::displayName($current_folder)) . ': <i>' . imap_last_error() . '</i>', HORDE_ERROR);
                            $return_value = false;
                        }
                    }
                    $msg_list = $msg_num;
                    imap_reopen($imp['stream'], $serverstring . $msg_folder);
                } else {
                    $msg_list .= ',' . $msg_num;
                }
                $current_folder = $msg_folder;
            }
            /* Tidy up the last few that haven't been copied/moved yet .. */
            if ($msg_list != '') {
                if (!$function($imp['stream'], $msg_list, $flag, ST_UID)) {
                    Horde::raiseMessage(sprintf(_("There was an error flagging messages in the folder \"%s\". This is what the server said"),
                                                IMP_folder::displayName($current_folder)) . ': <i>' . imap_last_error() . '</i>', HORDE_ERROR);
                    $return_value = false;
                }
            }
        }
        else {
            $msg_list = implode($msgs, ',');
            if (!$function($imp['stream'], $msg_list, $flag, ST_UID)) {
                Horde::raiseMessage(sprintf(_("There was an error flagging messages. This is what the server said")) .
                                    ': <i>' . imap_last_error() . '</i>', HORDE_ERROR);
                $return_value = false;
            }
        }
        
        return $return_value;
    }
    
}
?>
